#Author : Sean McKiernan (Mekire)
#Purpose: My chess clock broke, so I wrote a new one.
#License: Free for everyone and anything (no warranty expressed or implied).

import sys,os,pickle

import pygame
from pygame.locals import *

os.environ['SDL_VIDEO_CENTERED'] = '1'

pygame.init()

SCREENSIZE  = (1200,400)

##SCREEN = pygame.display.set_mode(SCREENSIZE,NOFRAME)
SCREEN = pygame.display.set_mode(SCREENSIZE,FULLSCREEN)

#graphics and fonts
numchart   = pygame.image.load("numchart.png").convert()
clockbg    = pygame.image.load("clockpic1.png").convert()
light      = pygame.image.load("indicator.png").convert_alpha()
delfield   = pygame.image.load("delfields.png").convert()
deltext    = pygame.image.load("dly.png").convert()
addtext    = pygame.image.load("add.png").convert()

NUMLIST = []
for j in range(2):
    for i in range(5):
        num = pygame.Surface((128,214)).convert()
        num.blit(numchart,(0,0),(i*114,j*200,128,214))
        NUMLIST.append(num)

SNUMLIST = []
for k in range(10):
    num = pygame.Surface((38,65)).convert()
    num.blit(numchart,(0,0),(k*35,415,38,65))
    SNUMLIST.append(num)

NUMBERKEYS = (K_1,K_2,K_3,K_4,K_5,K_6,K_7,K_8,K_9,K_0)
    
class Display:
    def __init__(self,time,location,specs=(0,0),dellocation = None):
        self.screen = SCREEN
        self.time = time
        self.location = location
        self.dellocation = dellocation
        self.setting = False
        self.increment = specs[1]

        #main timer counters
        self.timer_started = 0.0
        self.time_elapsed  = 0.0
        self.rollover = 0.0
        self.flash_update = 0.0

        self.active = False
        self.change = True
        
        self.dead   = False
        self.flash  = False

        #delay related counters
        self.delay = specs[0]
        self.delcount = self.delay
        self.del_started = 0.0
        self.del_elapsed = 0.0
        self.deldone = False
        self.delchange = True

    def update(self):
        self.pass_time()
        if self.change:
            self.minutes = str(self.time//60).zfill(2)
            self.seconds = str(self.time % 60).zfill(2)
            for x,digit in enumerate(self.minutes):
                if x == 0 and int(digit) == 0 and not self.setting:
                    self.screen.fill((0,0,0),(self.location[0],self.location[1],128,214))
                else:
                    self.screen.blit(NUMLIST[int(digit)],(self.location[0]+x*114,self.location[1]))                                 
            for y,digit in enumerate(self.seconds):
                self.screen.blit(NUMLIST[int(digit)],(self.location[0]+256+y*114,self.location[1]))
            self.change = False
            pygame.display.update()
        elif self.dead:
            if self.flash:
                if pygame.time.get_ticks() - self.flash_update > 1000.0/4:
                    self.screen.fill((0,0,0),(self.location,(498,214)))
                    self.flash_update = pygame.time.get_ticks()
                    self.flash = False
                    pygame.display.update()
            else:
                if pygame.time.get_ticks() - self.flash_update > 1000.0/4:
                    for x,digit in enumerate('00'):
                        self.screen.blit(NUMLIST[int(digit)],(self.location[0]+x*114,self.location[1]))
                        self.screen.blit(NUMLIST[int(digit)],(self.location[0]+256+x*114,self.location[1]))
                    self.screen.blit(clockbg,(self.location[0]+242,self.location[1]),(self.location[0]+242,self.location[1],14,214))               
                    self.flash_update = pygame.time.get_ticks()
                    self.flash = True
                    pygame.display.update()
        if self.delay and self.delchange:
            self.screen.blit(delfield,(self.dellocation[0]-2,self.dellocation[1]-2))
            for z,digit in enumerate(str(self.delcount).zfill(2)):
                self.screen.blit(SNUMLIST[int(digit)],(self.dellocation[0]+z*35,self.dellocation[1]))
            self.delchange = False
            pygame.display.update()
            
    def activate(self):
        self.active = True
        if not self.delay:
            self.timer_started = pygame.time.get_ticks()-self.rollover
        else:
            self.del_started = pygame.time.get_ticks()

    def inactivate(self):
        self.active = False
        if not self.delay or self.deldone:
            self.rollover = self.time_elapsed
            self.time_elapsed = 0.0
        if self.delay:
            self.delcount  = self.delay
            self.deldone   = False
            self.delchange = True
        if self.increment and not self.dead:
            self.time  += self.increment
            self.change = True

    def pass_time(self):
        if self.active:
            if not self.delay or self.deldone:
                if not self.dead:
                    self.time_elapsed = pygame.time.get_ticks()-self.timer_started
                    if not self.dead and self.time_elapsed >= 1000:
                        self.time -= 1
                        self.timer_started = pygame.time.get_ticks()
                        self.time_elapsed = 0.0
                        if self.time != 0:
                            self.change = True
                        else:
                            self.dead   = True
                            self.change = False
            else:
                self.del_elapsed = pygame.time.get_ticks()-self.del_started
                if self.del_elapsed >= 1000:
                    self.delcount -= 1
                    self.del_started = pygame.time.get_ticks()
                    self.del_elapsed = 0.0
                    if self.delcount != 0:
                        self.delchange = True
                    else:
                        self.delchange = True
                        self.deldone   = True
                        self.timer_started = pygame.time.get_ticks()-self.rollover
                   
class ChessClock:
    def __init__(self):
        self.screen = SCREEN
        myfile = open("presets.dat",'rb')
        self.presets = pickle.load(myfile)
        myfile.close()
##        self.factory_reset() #uncomment to restore 'factory' presets
        self.Lmin = 10; self.Rmin = 10
        self.Lsec = 0; self.Rsec = 0
        self.dly = 0
        self.inc = 0
        self.load_preset(self.presets[0])
        self.init()

    def init(self):
        self.state = "START"
        self.screen.blit(clockbg,(0,0))
        pygame.display.update()
        self.LDisplay = Display(self.Lmin*60+self.Lsec,( 35,100),(self.dly,self.inc),(435,23))
        self.RDisplay = Display(self.Rmin*60+self.Rsec,(667,100),(self.dly,self.inc),(692,23))
        self.LDisplay.update()
        self.RDisplay.update()
        
        self.spacecount = 5
        self.space_start = 0.0
        self.spacedown = False
        self.setindex  = 0

        self.saveindex  = 0
        self.savedown   = False
        self.save_start = 0.0

        self.which = (0,0)
        
    def event_loop(self):
        self.target = pygame.mouse.get_pos()
        for click in pygame.event.get():
            if click.type == MOUSEBUTTONDOWN:
                hit = pygame.mouse.get_pressed()
                if hit[0]:
                    if self.state in ("RUN","START","PAUSE"):
                        if self.LDisplay.active or (not self.RDisplay.active and not self.LDisplay.active):
                            self.screen.blit(light,(1110,40))
                            self.screen.blit(clockbg,(40,40),(40,40,50,50))
                            pygame.display.update()
                            self.RDisplay.activate()
                            self.spacecount = 0
                            if self.LDisplay.active:
                                self.LDisplay.inactivate()
                            if self.state in ["PAUSE","START"]:
                                for Dis in (self.LDisplay,self.RDisplay):
                                    self.screen.blit(clockbg,(Dis.location[0]+242,Dis.location[1]),
                                                    (Dis.location[0]+242,Dis.location[1],14,214))
                                if self.state == "PAUSE" and self.which[0]:
                                    self.LDisplay.delcount = self.dly
                                    self.LDisplay.delchange = True
                                    self.LDisplay.deldone   = False
                                self.which = (0,0)
                                self.state = "RUN"
                            
                    elif self.state == "SETTING":
                        self.change_left()
                        
                if hit[2]:
                    if self.state in ("RUN","START","PAUSE"):
                        if self.RDisplay.active or (not self.RDisplay.active and not self.LDisplay.active):
                            self.screen.blit(light,(40,40))
                            self.screen.blit(clockbg,(1110,40),(1110,40,50,50))
                            pygame.display.update()
                            self.LDisplay.activate()
                            self.spacecount = 0
                            if self.RDisplay.active:
                                self.RDisplay.inactivate()
                            if self.state in ["PAUSE","START"]:
                                for Dis in (self.LDisplay,self.RDisplay):
                                    self.screen.blit(clockbg,(Dis.location[0]+242,Dis.location[1]),
                                                    (Dis.location[0]+242,Dis.location[1],14,214))
                                if self.state == "PAUSE" and self.which[1]:
                                    self.RDisplay.delcount = self.dly
                                    self.RDisplay.delchange = True
                                    self.RDisplay.deldone   = False
                                self.which = (0,0)
                                self.state = "RUN"
                            
                    elif self.state == "SETTING":
                        self.change_right()
                        
            elif click.type == QUIT:
                self.state = "QUIT"

            elif click.type == KEYDOWN:
                if click.key == K_ESCAPE:
                    self.state = "QUIT"
                elif click.key == K_SPACE:
                    self.space_start = pygame.time.get_ticks()
                    self.spacedown   = True
                    self.spacecount += 1
                    if self.state == "SETTING":
                        self.spacecount = 5
                        if self.setindex < 5:
                            self.setindex += 1
                        else:
                            for Dis in (self.LDisplay,self.RDisplay):
                                self.screen.blit(clockbg,(Dis.location[0]+242,Dis.location[1]),
                                                (Dis.location[0]+242,Dis.location[1],14,214))
                                Dis.setting = False
                            if not self.dly:
                                self.screen.blit(clockbg,(433,21),(433,21,334,69))
                            self.setindex = 0
                            self.state = "START"
                    if self.spacecount == 1:
                        if self.LDisplay.active:
                            self.which = (1,0)
                        elif self.RDisplay.active:
                            self.which = (0,1)
                        self.LDisplay.active = False
                        self.RDisplay.active = False
                        self.LDisplay.flash = True
                        self.RDisplay.flash = True
                        self.LDisplay.flash_update = 0.0
                        self.RDisplay.flash_update = 0.0
                        self.state = "PAUSE"
                    elif self.spacecount == 3:
                        self.init()
                elif click.key in NUMBERKEYS and self.state == "START":
                    self.saveindex = NUMBERKEYS.index(click.key)
                    self.savedown  = True
                    self.save_start = pygame.time.get_ticks()
                    
                        
            elif click.type == KEYUP:
                if click.key == K_SPACE:
                    self.space_start = 0.0
                    self.spacedown = False
                elif click.key in NUMBERKEYS and self.state == "START":
                    index = NUMBERKEYS.index(click.key)
                    self.savedown = False
                    self.save_start = 0.0
                    try:
                        self.load_preset(self.presets[index])
                        self.init()
                    except IndexError:
                        print("Preset not found")

        if self.state == "START" and self.spacedown and (pygame.time.get_ticks() - self.space_start >= 1000):
            self.state = "SETTING"

        if self.state == "START" and self.savedown and  (pygame.time.get_ticks() - self.save_start >= 1000):
            self.state = "SAVE"
            for Dis in (self.LDisplay,self.RDisplay):
                Dis.flash = True
    
    def load_preset(self,preset):
        self.Lmin = preset[0]// 60 ; self.Rmin = preset[1]// 60
        self.Lsec = preset[0] % 60 ; self.Lsec = preset[1] % 60
        self.dly = preset[2]
        self.inc = preset[3]

    def save_preset(self):
        for Dis in (self.LDisplay,self.RDisplay):
            if Dis.flash:
                if pygame.time.get_ticks() - Dis.flash_update > 1000.0/4:
                    self.screen.fill((0,0,0),(Dis.location,(498,214)))
                    Dis.flash_update = pygame.time.get_ticks()
                    Dis.flash  = False
                    Dis.change = True
            else:
                if pygame.time.get_ticks() - Dis.flash_update > 1000.0/4:
                    self.screen.blit(clockbg,(Dis.location[0]+242,Dis.location[1]),(Dis.location[0]+242,Dis.location[1],14,214))
                    Dis.update()
                    Dis.flash_update = pygame.time.get_ticks()
                    Dis.flash  = True
                    self.presets[self.saveindex] = (self.Lmin*60+self.Lsec,self.Rmin*60+self.Rsec,self.dly,self.inc)
                    myfile = open("presets.dat",'wb')
                    pickle.dump(self.presets,myfile)
                    myfile.close()
                    self.state = "START"
                    self.savedown = False
                    self.save_start = 0.0
        pygame.display.update()

    def factory_reset(self):
        self.presets = [(300,300,0, 0),
                        (600,600,3, 0),
                        (900,900,0, 0),
                        (180,180,0, 0),
                        (120,120,0,12),
                        (),(),(),(),()]
        myfile = open("presets.dat",'wb')
        pickle.dump(self.presets,myfile)
        myfile.close()
                                 
    def set_it(self):
        for Dis in (self.LDisplay,self.RDisplay):
            Dis.setting = True
            if Dis.flash:
                if pygame.time.get_ticks() - Dis.flash_update > 1000.0/4:
                    if self.setindex in (0,1):
                        self.screen.fill((0,0,0),((Dis.location[0]+114*self.setindex,Dis.location[1]),(128,214)))
                    elif self.setindex in (2,3):
                        self.screen.fill((0,0,0),((Dis.location[0]+114*self.setindex+28,Dis.location[1]),(128,214)))
                    elif self.setindex in (4,5):
                        self.screen.fill((0,0,0),((Dis.location[0]+114*(self.setindex-2)+28,Dis.location[1]),(128,214)))
                        if Dis is self.LDisplay:
                            self.screen.blit(deltext,Dis.location)
                        elif Dis is self.RDisplay:
                            self.screen.blit(addtext,Dis.location)
                    Dis.flash_update = pygame.time.get_ticks()
                    Dis.flash  = False
                    Dis.change = True
            else:
                if pygame.time.get_ticks() - Dis.flash_update > 1000.0/4:
                    if self.setindex in (0,1,2,3):
                        self.screen.blit(clockbg,(Dis.location[0]+242,Dis.location[1]),(Dis.location[0]+242,Dis.location[1],14,214))
                        Dis.update()
                    elif self.setindex in (4,5):
                        if Dis is self.LDisplay:
                            for i,digit in enumerate(str(self.dly).zfill(2)):
                                self.screen.blit(NUMLIST[int(digit)],(Dis.location[0]+256+i*114,Dis.location[1]))
                        elif Dis is self.RDisplay:
                            for i,digit in enumerate(str(self.inc).zfill(2)):
                                self.screen.blit(NUMLIST[int(digit)],(Dis.location[0]+256+i*114,Dis.location[1]))                   
                    Dis.flash_update = pygame.time.get_ticks()
                    Dis.flash  = True
        pygame.display.update()

    def change_left(self):
        if self.setindex == 0:
            if self.LDisplay.time < 90*60:
                self.LDisplay.time += 10*60
                self.Lmin += 10
            else:
                self.LDisplay.time -= 90*60
                self.Lmin -= 90
        elif self.setindex == 1:
            if self.LDisplay.time % (10*60) < 9*60:
                self.LDisplay.time += 1*60
                self.Lmin += 1
            else:
                self.LDisplay.time -= 9*60
                self.Lmin -= 9
        elif self.setindex == 2:
            if self.LDisplay.time % 60 < 50:
                self.LDisplay.time += 10
                self.Lsec += 10
            else:
                self.LDisplay.time -= 50
                self.Lsec -= 50
        elif self.setindex == 3:
            if self.LDisplay.time % 10 < 9:
                self.LDisplay.time += 1
                self.Lsec += 1
            else:
                self.LDisplay.time -= 9
                self.Lsec -= 9
        elif self.setindex == 4:
            if self.dly < 50:
                self.dly += 10
            else:
                self.dly -= 50
            self.LDisplay.delay = self.dly
            self.RDisplay.delay = self.dly
            self.LDisplay.delcount = self.dly
            self.RDisplay.delcount = self.dly
            self.LDisplay.delchange = True
            self.LDisplay.deldone   = False
            self.RDisplay.delchange = True
            self.RDisplay.deldone   = False
        elif self.setindex == 5:
            if self.dly % 10 < 9:
                self.dly += 1
            else:
                self.dly -= 9
            self.LDisplay.delay = self.dly
            self.RDisplay.delay = self.dly
            self.LDisplay.delcount = self.dly
            self.RDisplay.delcount = self.dly
            self.LDisplay.delchange = True
            self.LDisplay.deldone   = False
            self.RDisplay.delchange = True
            self.RDisplay.deldone   = False

    def change_right(self):
        if self.setindex == 0:
            if self.RDisplay.time < 90*60:
                self.RDisplay.time += 10*60
                self.Rmin += 10
            else:
                self.RDisplay.time -= 90*60
                self.Rmin -= 90
        elif self.setindex == 1:
            if self.RDisplay.time % (10*60) < 9*60:
                self.RDisplay.time += 1*60
                self.Rmin += 1
            else:
                self.RDisplay.time -= 9*60
                self.Rmin -= 9
        elif self.setindex == 2:
            if self.RDisplay.time % 60 < 50:
                self.RDisplay.time += 10
                self.Rsec += 10
            else:
                self.RDisplay.time -= 50
                self.Rsec -= 50
        elif self.setindex == 3:
            if self.RDisplay.time % 10 < 9:
                self.RDisplay.time += 1
                self.Rsec += 1
            else:
                self.RDisplay.time -= 9
                self.Rsec -= 9
        elif self.setindex == 4:
            if self.inc < 50:
                self.inc += 10
            else:
                self.inc -= 50
            self.LDisplay.increment = self.inc
            self.RDisplay.increment = self.inc
        elif self.setindex == 5:
            if self.inc % 10 < 9:
                self.inc += 1
            else:
                self.inc -= 9
            self.LDisplay.increment = self.inc
            self.RDisplay.increment = self.inc

    def pause_it(self):
        for Dis in (self.LDisplay,self.RDisplay):
            if Dis.flash:
                if pygame.time.get_ticks() - Dis.flash_update > 1000.0/4:
                    self.screen.fill((0,0,0),(Dis.location,(498,214)))
                    Dis.flash_update = pygame.time.get_ticks()
                    Dis.flash  = False
                    Dis.change = True
            else:
                if pygame.time.get_ticks() - Dis.flash_update > 1000.0/4:
                    self.screen.blit(clockbg,(Dis.location[0]+242,Dis.location[1]),(Dis.location[0]+242,Dis.location[1],14,214))
                    Dis.update()
                    Dis.flash_update = pygame.time.get_ticks()
                    Dis.flash  = True
        pygame.display.update()
    
    def update(self):
        #control flow     
        if self.state in ["RUN","START"]:
            self.LDisplay.update()
            self.RDisplay.update()
        elif self.state == "PAUSE":
            self.pause_it()
        elif self.state == "SETTING":
            self.set_it()
        elif self.state == "SAVE":
            self.save_preset()
        elif self.state == "QUIT":
            pygame.quit();sys.exit()
        self.event_loop()
        pygame.event.pump()
                    
#####           
def main():
    MyClock.update()

#####
if __name__ == "__main__":
    MyClock = ChessClock()
    while 1:
        main()


                
